 // Model railroad traverser control code for Arduino
 // Version 1.1
 //
 // Features:
 // - Manual control buttons
 // - Acceleration and deceleration using the Accelstepper library
 // - Emergency stop function 
 // - DCC interface using NMRA library 
 //
 // (c) Jiri Zlamal, e-mail: jiri@zlamal.cz
 // January 2021
 // Creative Commons License BY-NC
 // For commercial usage, contact the autor.
 
 #include <AccelStepper.h>                          // Including the Accellstepper library
 #include <NmraDcc.h>                               // Including the NMRA library
 
                                                    // Input/output pins used:
 const int DIR_PIN = 4;                             // Direction pin for A4988 driver
 const int STEP_PIN = 5;                            // Step control pin for A4988 driver
 const int MS1_PIN = 8;                             // Microstepping control pin for A4988 driver (for future use)
 const int MS2_PIN = 7;                             // Microstepping control pin for A4988 driver (for future use)
 const int MS3_PIN = 6;                             // Microstepping control pin for A4988 driver (for future use)
 const int ENABLE_PIN = 9;                          // Enable pin for A4988 driver
 const int SENSOR_PIN = 10;                         // Limit switch - HOME position, active in HIGH state
 const int BUTTON_PIN[4] = {A0, A1, A2, A3};        // Buttons: POS0 (Home), POS1, POS2, POS3 
 const int STOP_PIN = A4;                           // Emergency switch - stops the motor if connected to GND (low)

                                                    // Values:
 const int TOTAL_POS = 3;                           // Total amount of working positions used;                                                   
 const int POS[4] = {-25000, 1000, 11133, 21266};   // Traverser positions in steps (microsteps), to be calculated manually
 const int MAX_SPEED = 2400;                        // Set the standard full speed of the traverser
 const int SPEED = 1200;                            // Set the speed for slow approach to the sensor
 const int ACCELERATION = 800;                      // Set the acceleration in steps/second^2
 int MS_MODE = 8;                                   // Default microstepping mode, fot future development
 int STOP_ACTIVE = HIGH;                            // Emergency stop input 
 const int BASE_DCC_ADDR = 200;                     // Base DCC address = position 0 address
 int posNumber = 0;                                 // Initial position;
 
 #define motorInterfaceType 1                       // Definition of how is the motor controlled, must be "1" if a driver is used



                                                    // Set up the AccelStepper object for the A4988 Stepper Motor Driver
 AccelStepper stepper = AccelStepper(motorInterfaceType, STEP_PIN, DIR_PIN);
 
 #define motorInterfaceType 1                       // Definition of how is the motor controlled, must be "1" if a driver is used

 
 NmraDcc  Dcc ;                                     // Library initialisation

 void setup() {                                     // Set up the parameters:
 stepper.setMaxSpeed(MAX_SPEED);                    // Stepper motor driver max speed (normal traverser speed)
 stepper.setAcceleration(ACCELERATION);             // Stepper moror driver acceleration
 
 Serial.begin(9600);                                // Enable serial communication for debugging
 Serial.println("Begin setup...");
 pinMode(SENSOR_PIN, INPUT);                        // Set up the digital input and output pins...
 pinMode(ENABLE_PIN, OUTPUT);
 pinMode(STEP_PIN, OUTPUT);
 pinMode(DIR_PIN, OUTPUT);
 pinMode(MS1_PIN, OUTPUT);
 pinMode(MS1_PIN, OUTPUT);
 pinMode(MS1_PIN, OUTPUT);
 pinMode(STOP_PIN, INPUT_PULLUP);                  
  
 for (int i=0; i<(TOTAL_POS +1); i++) {             // Set up the analogue push button pins using pullup resistors
 pinMode(BUTTON_PIN[i], INPUT_PULLUP);
 }                                                  // End of the pushbutton loop
 
 Serial.println("NMRA DCC initializing...");
 Dcc.pin(0, 2, 1);                                  // Set up which External Interrupt, the Pin it's associated with that we're using and enable the Pull-Up
 Dcc.init( MAN_ID_DIY, 10, CV29_ACCESSORY_DECODER | CV29_OUTPUT_ADDRESS_MODE, 0 );   // Call the main DCC Init function to enable the DCC Receiver
 Serial.println("Init Done");

 //moveToHome();                                    // Move the traverser to the home position after rebooting/powering up.
                                                    // Uncomment this function if it is required to move the traverser home after rebooting
 }                                                  // End of the Setup void


                                                    // This function is called whenever a normal DCC Turnout Packet is received
 void notifyDccAccTurnoutOutput( uint16_t Addr, uint8_t Direction, uint8_t OutputPower ) 
 {
 Serial.print("notifyDccAccTurnoutOutput: ") ;      // Print the received packet values...
 Serial.print(Addr,DEC) ;
 Serial.print(',');
 Serial.print(Direction,DEC) ;
 Serial.print(',');
 Serial.println(OutputPower, HEX) ;

 posNumber = Addr - BASE_DCC_ADDR;                  // Get the position number from the received address
 if (posNumber == 0) {                              // Check if the required position is 0 (Home)...  
  moveToHome();                                     // Call the moveToHome function to move to the HOME position.
  }                                      
 else if 
 ((posNumber > 0)&&(posNumber < (TOTAL_POS +1))) {  // Check if the posNumber is within the allowed range to control the traverser
  moveToPosition (posNumber);                       // Call the moveToPosition function to move to the selected track.
  }
 }                                                  // End of void ==================================================

 

 int pollButtons() {                                // Function by Michael Hardwick
                                                    // To debounce the button input signals by repeated reading of their status
 for (int i=0; i<4; i++) {                          // Cycle through the buttons
 if (!digitalRead(BUTTON_PIN[i])) {                 // Look closer if the button appears pushed
 int counter = 0;                                   // If yes, set a counter to zero
 while (!digitalRead(BUTTON_PIN[i])) {              // Check the button every 10 ms for 100 ms
 if (counter == 10){                                // If the button was checked 10 times, it is pressed
 return i;                                          // So return the button index
 }
 counter++;                                         // Increment the counter  how many times was the button checked
 delay(10);                                         // Delay 10 milliseconds
 }
 }
 }                                                  // End of button loop
 return -1;                                         // No button was pressed, so return -1. Necessary for correct operation!
 }                                                  // End of void ==================================================


 void moveToHome() {                                // This routine moves the traverser to the home position in two phases:
 
 digitalWrite (ENABLE_PIN, LOW);                    // Enable the stepper motor
 if(!digitalRead(SENSOR_PIN)) {                     // If the limit sensor is not active (i.e. the traverser is not at home)...
 stepper.moveTo (POS[0]);                           // Move to a distant position behind the sensor
 while(!digitalRead(SENSOR_PIN)) {                  // While the limit switch (sensor) is not reached...
  if (digitalRead(STOP_PIN)) {                      // Check for the Emergency Stop signal (button or command) after each step
   Serial.println("EMERGENCY STOP");                // Print a message if the STOP function is activated
   stepper.stop ();                                 // Stop the motor...
   stepper.setCurrentPosition(stepper.currentPosition ()); // immediately by setting the current position as a target position
   digitalWrite (ENABLE_PIN, HIGH);                 // Disable the stepper motor driver
  return;                                           // Exit the loop if the stop function is activated       
 } 
 stepper.run();                                     // Run the motor until the sensor is reached...
 }
 stepper.stop ();                                   // Then stop the traverser...
 stepper.setCurrentPosition(0);                     // and reset the stepper motor position counter to zero
 }
 stepper.moveTo (400);                              // Move a bit back... (This is done anyway, even if the traverser is at home, to ensure the position is correct
 stepper.runToPosition();                           // Run there
 stepper.moveTo (-800);                             // And go again towards the sensor
 while(!digitalRead(SENSOR_PIN)) {stepper.run();}   // Run the motor until the sensor is reached again
 stepper.stop ();                                   // Then stop the traverser...
 stepper.setCurrentPosition(0);                     // and reset the stepper motor position counter to zero
 digitalWrite (ENABLE_PIN, HIGH);                   // Disable the stepper motor driver
 Serial.println("Traverser is at HOME position.");  // Print a message 
 }                                                  // End of void ==================================================


 void moveToPosition(int posNumber) {               // This routine moves the traverser into the selected position
  
 Serial.print("Traverser moves to POS ");           // Print out where is the traverser going
 Serial.println(posNumber);                         // The position is defined by the button pressed 
 digitalWrite (ENABLE_PIN, LOW);                    // Enable the motor driver 
 stepper.moveTo (POS[posNumber]);                   // Run to the selected position
  while (stepper.distanceToGo () != 0 ) {           // While the final position is not reached...
   if (digitalRead(STOP_PIN)) {                     // check for the Emergency Stop signal (button or command) after each step
   Serial.println("EMERGENCY STOP");                // Print a message if the STOP function is activated
   stepper.stop ();                                 // Stop the motor...
   stepper.setCurrentPosition(stepper.currentPosition ());  // immediately by setting the current position as a target position
  }                     
 stepper.run ();                                    // If the Emergency stop is not active, run normally to the desired position
 }
 digitalWrite (ENABLE_PIN, HIGH);                   // Disable the stepper motor driver
 Serial.print("Position ");                         // Print a message that the position was reached
 Serial.print (posNumber);
 Serial.println (" reached");  
 }                                                  // End of void ==================================================

                                                    
 void loop() {                                      // Main loop 
  
 Dcc.process();                                     // Call the DCC process
  
                                                    // Poll for button presses. If a button is pressed, go home if the home button is pressed
                                                    // or go to selected position
                                                    
 int posNumber = pollButtons();                     // Returns the button number (0-3)
 if (posNumber == 0) {                              // HOME button was pressed
  Serial.println("Home position selected");         // Print a message 
  moveToHome();                                     // Call the moveToHome function to move to the HOME position.
  }
  else if (posNumber > 0) {                         // Track position button was pressed
  Serial.print("Position ");                        // Print a message which position was selected
  Serial.print(posNumber);
  Serial.println(" selected");
  moveToPosition(posNumber);                        // Call the moveToPosition function to move to the selected track
  }                                                 // End of void ==================================================
 
 }                                                  // End of code  


 
 
